﻿using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using System;
using System.Collections.Generic;
using System.Linq;

using TestProject.Data;
using TestProject.Models;

namespace TestProject
{
    /// <summary>
    /// 커스텀 구성 공급자
    /// </summary>
    public class CustomConfigurationProvider : ConfigurationProvider
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 옵션 액션 - OptionsAction

        /// <summary>
        /// 옵션 액션
        /// </summary>
        Action<DbContextOptionsBuilder> OptionsAction { get; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomConfigurationProvider(optionsAction)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="optionsAction"></param>
        public CustomConfigurationProvider(Action<DbContextOptionsBuilder> optionsAction)
        {
            OptionsAction = optionsAction;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 로드하기 - Load()

        /// <summary>
        /// 로드하기
        /// </summary>
        public override void Load()
        {
            DbContextOptionsBuilder<DatabaseContext> builder = new DbContextOptionsBuilder<DatabaseContext>();

            OptionsAction(builder);

            using(DatabaseContext context = new DatabaseContext(builder.Options))
            {
                context.Database.EnsureCreated();

                if(!context.Configuration.Any())
                {
                    Data = CreateData(context);
                }
                else
                {
                    Data = context.Configuration.ToDictionary
                    (
                        configuration => configuration.ID,
                        configuration => configuration.Value
                    );
                }
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 데이터 생성하기 - CreateData(context)

        /// <summary>
        /// 데이터 생성하기
        /// </summary>
        /// <param name="context">데이터베이스 컨텍스트</param>
        /// <returns>데이터</returns>
        private static IDictionary<string, string> CreateData(DatabaseContext context)
        {
            Dictionary<string, string> sourceDictionary = new Dictionary<string, string>(StringComparer.OrdinalIgnoreCase)
            {
                { "Key1", "Value1" },
                { "Key2", "Value2" },
                { "Key3", "Value3" }
            };

            context.Configuration.AddRange
            (
                sourceDictionary.Select
                (
                    keyValuePair => new Configuration 
                    {
                        ID    = keyValuePair.Key,
                        Value = keyValuePair.Value
                    }
                )
                .ToArray()
            );

            context.SaveChanges();

            return sourceDictionary;
        }

        #endregion
    }
}